#region Copyright Syncfusion Inc. 2001 - 2014
// Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
// Use of this code is subject to the terms of our license.
// A copy of the current license can be obtained at any time by e-mailing
// licensing@syncfusion.com. Any infringement will be prosecuted under
// applicable laws. 
#endregion
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.Threading.Tasks;
using System.Windows.Threading;

namespace SelfRelationalDatabindingDemo
{
    public delegate void CollectionUpdatedEventHandler(object sender, EventArgs e);

    class ViewModel
    {
        event CollectionUpdatedEventHandler CollectionUpdated;
        private static Random random = new Random(123123);
        /// <summary>
        /// Initializes a new instance of the <see cref="ViewModel"/> class.
        /// </summary>
        public ViewModel()
        {
            this.PersonDetails2 = new ObservableCollection<PersonInfo>();
            this.CollectionUpdated += ViewModel_CollectionUpdated;
            this.PersonDetails = new ObservableCollection<PersonInfo>();
            this.PersonDetails.CollectionChanged += PersonDetails_CollectionChanged;
            Task[] tasks = new Task[1];
            tasks[0] = Task.Factory.StartNew(async () => await this.getSource());
            //Task.WaitAll(tasks);
            //getSource();
            
        }

        void ViewModel_CollectionUpdated(object sender, EventArgs e)
        {
            foreach(PersonInfo p in PersonDetails)
            {
                if (!PersonDetails2.Contains(p))
                    PersonDetails2.Add(p);
            }
        }

        void PersonDetails_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            Console.WriteLine("Changed");
        }

        async public Task<bool> getSource()
       {
           this.PersonDetails = await CreateGenericPersonData(5, 8);
            if(this.CollectionUpdated != null)
                CollectionUpdated(this, new EventArgs());
           return true;
       }
        

        private async Task<Task> Init(Double p)
        {
            var tcs = new TaskCompletionSource<bool>();
            System.Timers.Timer timer = new System.Timers.Timer();
            timer.Elapsed += (obj, args) =>
            {
                tcs.TrySetResult(true);
            };
            timer.Interval = p;
            timer.AutoReset = false;
            timer.Start();
            return tcs.Task;
        }


        private ObservableCollection<PersonInfo> _PersonDetails;
       
        /// <summary>
        /// Gets or sets the person details.
        /// </summary>
        /// <value>The person details.</value>
        public ObservableCollection<PersonInfo> PersonDetails
        {
            get { return _PersonDetails; }
            set { _PersonDetails = value; }
        }

        public ObservableCollection<PersonInfo> PersonDetails2 { get; private set; }

        /// <summary>
        /// Initializes a new instance of the <see cref="ViewModel"/> class.
        /// </summary>
        /// <param name="count">The count.</param>
        /// <param name="maxGenerations">The max generations.</param>
        public ViewModel(int count, int maxGenerations)
        {
            CreateGenericPersonData(count, maxGenerations);
        }

        /// <summary>
        /// Creates the child list.
        /// </summary>
        /// <param name="count">The count.</param>
        /// <param name="maxGenerations">The max generations.</param>
        /// <param name="lastName">The last name.</param>
        /// <returns></returns>
        public async Task <ObservableCollection<PersonInfo>> CreateChildList(int count, int maxGenerations, string lastName)
        {
            ObservableCollection<PersonInfo> _children = new ObservableCollection<PersonInfo>();
            
            if (count > 0 && maxGenerations > 0)
            {
                _children =await CreateGenericPersonData(count, maxGenerations - 1);
                foreach (PersonInfo p in _children)
                    p.LastName = lastName;
            }
            return _children;
        }

        /// <summary>
        /// Creates the generic person data.
        /// </summary>
        /// <param name="count">The count.</param>
        /// <param name="maxGenerations">The max generations.</param>
        /// <returns></returns>
        public async Task<ObservableCollection<PersonInfo>> CreateGenericPersonData(int count, int maxGenerations)
        {
            var personList = new ObservableCollection<PersonInfo>();

            if (count > 0)
            {
                Task.Delay(20).Wait();
                for (int i = 0; i < count; i++)
                {
                    var lastname = names2[random.Next(names2.GetLength(0))];
                   
                    personList.Add(new Person()
                    {
                        FirstName = names1[random.Next(names1.GetLength(0))],
                        LastName = lastname,
                        Children = await this.CreateChildList(count, (int)Math.Max(0, maxGenerations - 1), lastname),
                        MyEyeColor = color[random.Next(color.GetLength(0))],
                        DOB = GenerateRandomDate()
                    });
                }
            }
            return personList;
        }


        //public async Task<DispatcherTimer> Schedule(Action _oncompleteion, int durationMS)
        //{
        //    DispatcherTimer timer = new DispatcherTimer();
        //    timer.Interval = TimeSpan.FromMilliseconds(durationMS);
        //    EventHandler<object> eh = (delegate(object sender, object e)
        //    {
        //        timer.Stop();
        //    });
        //    timer.Tick += eh;
        //    sleep(2000);
        //    _oncompleteion();
        //    return timer;
        //}
        /// <summary>
        /// Generates the random date.
        /// </summary>
        /// <returns></returns>
        private DateTime GenerateRandomDate()
        {
            int randInt = random.Next(4000);
            return DateTime.Now.AddDays(-8000 + randInt);
        }

        #region Array Collection

        private static string[] names1 = new string[]{
            "George","John","Thomas","James","Andrew","Martin","William","Zachary",
            "Millard","Franklin","Abraham","Ulysses","Rutherford","Chester","Grover","Benjamin",
            "Theodore","Woodrow","Warren","Calvin","Herbert","Franklin","Harry","Dwight","Lyndon",
            "Richard","Gerald","Jimmy","Ronald","George","Bill", "Barack", "Warner","Peter", "Larry"
        };
        private static string[] names2 = new string[]{
             "Washington","Adams","Jefferson","Madison","Monroe","Jackson","Van Buren","Harrison","Tyler",
             "Polk","Taylor","Fillmore","Pierce","Buchanan","Lincoln","Johnson","Grant","Hayes","Garfield",
             "Arthur","Cleveland","Harrison","McKinley","Roosevelt","Taft","Wilson","Harding","Coolidge",
             "Hoover","Truman","Eisenhower","Kennedy","Johnson","Nixon","Ford","Carter","Reagan","Bush",
             "Clinton","Bush","Obama","Smith","Jones","Stogner"
        };
        private static string[] color = new string[]{
            "Red", "Blue", "Brown", "Unknown"
        };
        #endregion
    }
}
